'------------------------------------------------------------------------------
' This program is Copyright (c) 2005 Microsoft Corporation.
'
' All rights reserved.
'
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
' ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
' THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
' PARTICULAR PURPOSE.
'
' IN NO EVENT SHALL MICROSOFT AND/OR ITS RESPECTIVE SUPPLIERS BE
' LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
' DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
' WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
' ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
' OF THIS CODE OR INFORMATION.
'------------------------------------------------------------------------------
'
'     ISA Server 2004 (Objets COM FPC)
'
'     Cache update for objects matching a given URL - v1.9
'
'     Concept : If object is in cache, it has been requested so is in log !
'               Read log for all URLs matching the given root URL
'               Retrieve in array or file all exact URL for matching objects
'               Update those objects in cache
'
'     Cmd line Arguments : /u : Root URL to update (URL used by ISA)
'                          /all : refresh cache for all published sites
'
'                          /r : File name for reporting (None = no file)
'                          /l : File name for matching objects
'                               (Def = cache.txt, None = no file)
'                          /w : File name for URL list (work with /all)
'                               (Def = yyymmdd_Published_Web_Sites.txt)
'                          /m : Max nb of records for reading (def = 10000)
'                          /d : Max nb of Days to analyse in log (def = 2)
'
'                          /h or /? or /help : Display help
'
'     Microsoft Consulting Services France - Jerome Cornier - 2005/02
'------------------------------------------------------------------------------

Option Explicit

Dim objFPCRoot  ' The FPCLib.FPC root object
Dim iErrorNumber
Dim strErrorDescription

Dim bDisplay
Dim arrObjectInLog(), arrFetchURL()
Dim iDistinctObject, iDistinctURL

' Create the root object for FPC (ISA) access
Set objFPCRoot = CreateObject("FPC.Root")

bDisplay = TRUE

Main

'------------------------------------------------------------------------------
'    Main Procedure
'------------------------------------------------------------------------------
Sub Main()
    Dim objArgs
    Dim iArgs, iNbArgs
    Dim bURLok
    Dim strCurrDir, strCoreFileName, strFileName, strQuestion
    Dim iURL, iObject
    Dim strLogString
    Dim StartProcess
    Dim oFileCache
    Dim TxtCache
    Dim strUrlRoot, strUrlSource, strUrlName
    Dim strCacheFile, strWebPubFile
    Dim iRecords, iDays
    
    'Default values
    strFileName = strDate() & "_Cache_Update.txt"
    strCacheFile = "cache.txt"
    strWebPubFile = strDate() & "_Published_Web_Sites.txt"
    iRecords = 10000
    iDays = 2

    Set objArgs = WScript.Arguments
    iNbArgs = (objArgs.Count - 1)

    '--------------------------------------------------------------------------
    ' Set values in command line mode
    '--------------------------------------------------------------------------
    If iNbArgs >= 0 Then
    
        bDisplay = FALSE
        bURLok = FALSE

        For iArgs = 0 to iNbArgs
            Select Case LCase(objArgs(iArgs))
                Case "/?", "-h", "/help", "-help" ' Help
                    call DisplayHelp()
                Case "/u", "-u" ' Url to Fetch
                    If (iArgs + 1) <= iNbArgs Then
                        strUrlRoot = objArgs(iArgs + 1)
                        bURLok = TRUE
                    Else
                        call DisplayHelp()
                    End If
                Case "/all", "-all" ' All web sites
                    strUrlRoot = "All_Published_Web_Sites"
                    bURLok = TRUE
                Case "/r", "-r" ' Report File
                    If (iArgs + 1) <= iNbArgs Then
                        strFileName = objArgs(iArgs + 1)
                    Else
                        call DisplayHelp()
                    End If
                Case "/l", "-l" ' Objects founds in log File
                    If (iArgs + 1) <= iNbArgs Then
                        strCacheFile = objArgs(iArgs + 1)
                    Else
                        call DisplayHelp()
                    End If
                Case "/w", "-w" ' List of all published URLs file
                    If (iArgs + 1) <= iNbArgs Then
                        strWebPubFile = objArgs(iArgs + 1)
                    Else
                        call DisplayHelp()
                    End If
                Case "/m", "-m" ' Max Records
                    If (iArgs + 1) <= iNbArgs Then
                        iRecords = CInt(objArgs(iArgs + 1))
                    Else
                        call DisplayHelp()
                    End If
                Case "/d", "-d" ' Days to analyse in Logs
                    If (iArgs + 1) <= iNbArgs Then
                        iDays = CInt(objArgs(iArgs + 1))
                    Else
                        call DisplayHelp()
                    End If
            End Select
        Next
    
        If bURLok = FALSE Then
            call DisplayHelp()
        End If

    Else
    '--------------------------------------------------------------------------
    ' If no arguments in line, set values from dialog boxes ;-)
    '--------------------------------------------------------------------------
        strCoreFileName = "_Cache_Update.txt"
        strFileName = InputBox ("Report file: " & _
            VbCrLf & VbCrLf & _
            "(This script can run in cmd line mode" & _
            " - /h to list options)", _
            "Enter a name for report file: ", _
            strDate() & strCoreFileName)
        If strFileName = "" Then
            call ProcessCanceled("Script canceled by user")
        End If

        ' URL to update (ex: "http://www.internalweb.com")
        strUrlRoot = InputBox ("Root URL to update (Internal): " & VbCrLf & _
            "(All_Published_Web_Sites to refresh entire cache)", _
            "Enter a root URL: ", "http://")
        If strUrlRoot = "" Then
            call ProcessCanceled("Script canceled by user")
        End If
        
        ' File to store matching objects found in log
        strCacheFile = InputBox ("Matching Object File: ", _
            "Enter a name for the matching object file:", strCacheFile)
        If strCacheFile = "" Then
            call ProcessCanceled("Script canceled by user")
        End If

        ' File to store Published Web Sites root URLs
        If strUrlRoot = "All_Published_Web_Sites" Then
            strWebPubFile = InputBox ("Published Web Sites URLs File: ", _
                "Enter a name for Published Web Sites URLs file:", strWebPubFile)
            If strWebPubFile = "" Then
                call ProcessCanceled("Script canceled by user")
            End If
        End If

        ' Max records nb
        iRecords = CInt(InputBox ("Max nb of records: ", _
            "Enter the max nb of records to read:", iRecords))
        If iRecords = 0 Then
            call ProcessCanceled("Script canceled by user")
        End If

        ' Max nb of days to analyze
        iDays = CInt(InputBox ("Nb of days: ", _
            "Enter the nb of days to analyze in log:", iDays))
        If iRecords = 0 Then
            call ProcessCanceled("Script canceled by user")
        End If
    End If

    '--------------------------------------------------------------------------
    ' Start of automatic process
    '--------------------------------------------------------------------------
    StartProcess = Date()

    ' URL name normalization (ex: http://www.web1.com)
    '--------------------------------------------------------------------------
    If LCase(strUrlRoot) <> "all_published_web_sites" Then
        If Left(strUrlRoot, 8) = "https://" Then
            strUrlRoot = Right(strUrlroot, Len(strUrlRoot) - 8)
        End If
        If Left(strUrlRoot, 7) <> "http://" Then
            strUrlRoot = "http://" & strUrlRoot
        End if  
        WScript.Echo strUrlRoot
        If Right(strUrlRoot, 1) = "/" Then
            strUrlRoot = Left(strUrlRoot, Len(strUrlRoot) - 1)
        End If
    End If

    ' Current path initialization (strCurrDir)
    '--------------------------------------------------------------------------
    Set oFileCache = WScript.CreateObject("Scripting.FileSystemObject")
    strCurrDir = oFileCache.GetAbsolutePathName(".")
    Set oFileCache = Nothing

    ' Report file initialization (erase if exist, no file created if "none")
    '--------------------------------------------------------------------------
    strFileName = strCurrDir & "\" & strFileName
    call LogAction(strFileName, "ISA Server 2004 - Cache Update", TRUE)
    call LogAction(strFileName, "Root URL to Update : " & strUrlRoot, FALSE)
    call LogAction(strFileName, VbCrLf & Date() & " " & Time() & VbCrLf, FALSE)

    ' Cache File Name initialization
    strCacheFile = strCurrDir & "\" & strCacheFile

    '__________________________________________________________________________
    '______________________________ Core ______________________________________

    '- Step 1 (Optional) ------------------------------------------------------
    ' list all root URL in ISA Web Publishing Rules if
    ' given URL = All_Published_Web_Sites
    '--------------------------------------------------------------------------
    If LCase(strUrlRoot) = "all_published_web_sites" Then
        ReDim arrFetchURL(5000)

        ' Web Sites URLs file initialization
        strWebPubFile = strCurrDir & "\" & strWebPubFile
        strLogString = "Published Web Sites : " & " Date : " & Date()
        call LogAction(strWebPubFile, strLogString, TRUE)

        ' List all published web sites
        call ListAllPublishedWebSites(objFPCRoot, strWebPubFile, strFileName)
    Else
        ReDim arrFetchURL(2)

        iDistinctURL = 1
        arrFetchURL(iDistinctURL) = strUrlRoot
    End If
    
    '- Step 2 -----------------------------------------------------------------
    ' Search in log for objects matching the given root URL
    ' ISA records in log the internal URL used to get site's objects
    '--------------------------------------------------------------------------
    For iURL = 1 to iDistinctURL
    
        strUrlRoot = arrFetchURL(iURL)

        ' Cache objects file initialization
        strLogString = "Objects list matching : " & _
            strUrlRoot & " Date : " & Date()
        call LogAction(strCacheFile, strLogString, TRUE)

        ' Search for Distincts Object's URLs
        call CacheRetrieveInLog(objFPCRoot, strUrlRoot, strCacheFile, _
            strFileName, iRecords, iDays)

        ' Log end of search Time
        strLogString = VbCrLf & Date() & " " & Time() & VbCrLf
        call LogAction(strFileName, strLogString, FALSE)

        '- Step 3 -------------------------------------------------------------
        ' Update line by line from an array in memory if name of file = None
        '----------------------------------------------------------------------
        If LCase(Right(strCacheFile, 4)) = "none" Then
            For iObject = 1 to iDistinctObject
                strUrlSource = arrObjectInLog(iObject)
                strUrlName = strUrlSource
                If Left(strUrlSource, 7) = "http://" Then

                    ' Cache update
                    call CacheUpdate(objFPCRoot, strUrlSource, strUrlName, strFileName)

                End If
            Next
        Else
        '----------------------------------------------------------------------
        ' If valid file name for cache exist proceed line by line reading the file
        '----------------------------------------------------------------------
            Set oFileCache = WScript.CreateObject("Scripting.FileSystemObject")
            Set TxtCache = oFileCache.OpenTextFile(strCacheFile, 1, True)

            ' Update Cache line by line
            Do While Not txtCache.AtEndOfStream
                strUrlSource = txtCache.ReadLine
                strUrlName = strUrlSource
                If Left(strUrlSource, 7) = "http://" Then

                    ' Cache update
                    call CacheUpdate(objFPCRoot, strUrlSource, strUrlName, strFileName)

                End If
            Loop

            ' close the file object
            TxtCache.close
            Set oFileCache = Nothing
        End If
    Next
    
    '_____________________________ End Core ___________________________________
    '__________________________________________________________________________
    
    ' write end-of-report informations
    strLogString = VbCrLf & Date() & " " & Time()
    'WScript.Echo strLogString
    call LogAction(strFileName, strLogString, FALSE)
    
    If bDisplay Then
        WScript.Echo "Fin"
    End If
End Sub

'------------------------------------------------------------------------------
'                         End of main procedure
'                   start of procedures & functions
'------------------------------------------------------------------------------

'----------------------------------------------------------------
'    Purpose : display a message, then close script
'    Arguments : string of the message to be displayed
'----------------------------------------------------------------
Sub ProcessCanceled(StrQuestion)

    MsgBox strQuestion,vbExclamation + vbOkOnly + vbApplicationModal + 256, _
        "End of Script"
    WScript.Quit(1)
End Sub

'----------------------------------------------------------------
'    Purpose : Process current date to retrieve a string
'    Return : YearMonthDay, format "yyyymmdd"
'----------------------------------------------------------------
Function strDate()
    Dim strToday, strMonth, strDay, strYear

    strToday = Date()
    strMonth = Month(strToday)
    strDay = Day(strToday)
    If Len(strMonth) < 2 Then strMonth = "0" & strMonth End If
    If Len(strDay) < 2 Then strDay = "0" & strDay End If
    strDate = Year(strToday) & strMonth & strDay
End Function

'----------------------------------------------------------------
'    Purpose : display help informations, then close the script
'----------------------------------------------------------------
Sub DisplayHelp()

    WScript.Echo "ISA Server - Cache Update for a given root URL" & _
    VbCrLf & _
    VbCrLf & "     Arguments :" & _
    VbCrLf & "          /u <root URL to update> (URL used by ISA)" & _
    VbCrlF & "          /all (refresh cache for all published web sites)" & _
    VbCrLf & "               (/all superceeds /u & /c)" & _
    VbCrLf & _
    VbCrLf & "         Optional :" & _
    VbCrLf & "          /r <Name of report file> (No report if " & "None" & ")" & _
    VbCrLf & "          /l <Name of objects list file> (default : cache.txt)" & _
    VbCrLf & "             (If None, the script use memory)" & _
    VbCrLf & "          /w <name of URLs list> (work with /all)" & _
    VbCrLf & "             (Def = yyymmdd_Published_Web_Sites.txt)" & _
    VbCrLf & "             (If None, the script use memory)" & _
    VbCrLf & "          /m <Nb of records to read> (default = 10000)" & _
    VbCrLf & "          /d <Nb of days to analyse in Logs> (default = 2)"
    
    WScript.Quit(-1)
End Sub

'----------------------------------------------------------------
'    Purpose : create or append lines in given file
'    Arguments : Name of File
'                String to write as a new line
'                create or append - True to create
'
'    No log if File Name = "None"
'----------------------------------------------------------------
Sub LogAction(strFileLog, strStringToLog, bCreate)
    Dim oFileLog
    Dim TxtLog

    ' Direct return if name-of-file is None
    If LCase(Right(strFileLog, 4)) = "none" Then
        Exit Sub
    End If

    ' Open file
    Set oFileLog = WScript.CreateObject("Scripting.FileSystemObject")
    
    If bCreate = TRUE Then
        Set TxtLog = oFileLog.OpenTextFile (strFilelog, 2, True)
    Else
        Set TxtLog = oFileLog.OpenTextFile (strFileLog, 8, True)
    End If
    
    ' Update log file
    TxtLog.Writeline strStringToLog

    'close file object
    TxtLog.close
    Set oFileLog = Nothing
 End Sub

'--------------------------------------------------------------
' Purpose : look for an existing line in a file
' Return : 0 Not Found
'          1 Found
'--------------------------------------------------------------
Function LineExistInFile(strFileToSearchInto, strLineToSearch)
    Dim oFileToOpen, txtToOpen

    LineExistInFile = 0

    ' open file
    Set oFileToOpen = WScript.CreateObject("Scripting.FileSystemObject")
    Set txtToOpen = oFileToOpen.OpenTextFile(strFileToSearchInto, 1, True)

    ' search for the given line
    Do While Not txtToOpen.AtEndOfStream
        If txtToOpen.ReadLine = strLineToSearch Then
            LineExistInFile = 1
            Set oFileToOpen = Nothing
            txtToOpen.close
            Exit Function
        End If
    Loop

    ' close file object
    Set oFileToOpen = Nothing
    txtToOpen.close
End Function

'--------------------------------------------------------------
' Purpose : Look for a line in array (global defined)
' Return : 0 Not Found
'          1 Found
'--------------------------------------------------------------
Function lineExistInArray(ByRef arrToSearchInto, strLineToSearch)
    Dim iLoop

    lineExistInArray = 0

    ' search line in array
    For iLoop = 1 to UBound(arrToSearchInto)
        If arrToSearchInto(iLoop) = strLineToSearch Then
            lineExistInArray = 1
            Exit Function
        End If
    Next
End Function

'------------------------------------------------------------
'    Purpose : List all Published Web Sites root URLs
'
'    Arguments : FPC (ISA) Root Object
'                string name of file to records URLs
'                strinf name of report file
'
'    Global : Error Code - 0 if ok
'             Error Description
'             Array to store URL for Internal Web Sites
'             Distinct URL Counter for rules array
'------------------------------------------------------------
Sub ListAllPublishedWebSites(ObjFPC, strFileRules, strFileLog)

    'Const for Rule types
    Const fpcPolicyRuleAccess = 0
    Const fpcPolicyRuleServerPublishing = 1
    Const fpcPolicyRuleWebPublishing = 2

    Dim array
    Dim fpcRules
    Dim fpcRule
    Dim strLogString
    Dim strWebSite
    Dim bURLok
    
    iDistinctURL = 0
    bURLok = FALSE

    On Error Resume Next

    ' Get references to the array object and the Rules collection.
    Set array = ObjFPC.GetContainingArray
    Set fpcRules = array.ArrayPolicy.PolicyRules

    ' List all PublicNames (cached URLs) if this is a Web Publishing Rule
    ' and the rule is Enable
    For Each fpcRule in fpcRules
        If fpcRule.Type = fpcPolicyRuleWebPublishing Then
            If fpcRule.Enabled Then
                strWebSite = fpcRule.WebPublishingProperties.WebSite
      	
                If LCase(Right(strFileRules, 4)) <> "none" Then
                    If LineExistInFile(strFileRules, strWebSite) = 0 Then
                        call LogAction(strFileRules, strWebSite, FALSE)
                    End If
                End If
                
                If lineExistInArray(arrFetchURL, strWebSite) = 0 Then
                    iDistinctURL = iDistinctURL + 1
                    arrFetchURL(iDistinctURL) = strWebSite
                End If
                
            End If
        End If
    Next
    
    If Err.Number <> 0 Then
        iErrorNumber = Err.Number
        strErrorDescription = Err.Description

        strLogString = "    Error nb : " & iErrorNumber & _
            VbCrLf & strFileRules & _
            VbCrLf & "    " & strErrorDescription
        call LogAction(strFileLog, strLogString, FALSE)
        If bDisplay Then
            WScript.Echo strLogString
        End If
    End If

End Sub

'------------------------------------------------------------
'    Purpose : Search all matching objects, then records them
'
'    Arguments : FPC (ISA) Root Object
'                string URL Racine
'                string Cache File Name
'                string Log File Name
'                Max records to retrieve
'                Max Days to analyse in Logs
'
'    Global : Error Code - 0 if ok
'             Error Description
'             Distinct Object Counter for cahce objects array
'------------------------------------------------------------
Sub CacheRetrieveInLog(ObjFPC, strUrlRacine, strFileCache, strFileLog, _
    iNbRecords, iNbDays)
    Dim ObjLogMSDE, ObjFilterExpressions
    Dim LogMSDERecord
    Dim strLogURLString, strLogString
    Dim iCounter
    Dim bObjectOk

    iCounter = 0
    iDistinctObject = 0
    iErrorNumber = 0

    ' values control
    If iNbRecords < 500 Then
        iNbRecords = 500
    End If
    If iNbDays < 1 Then
        iNbDays = 1
    End If

    ReDim arrObjectInLog(iNbRecords)
    
    On Error Resume Next

    ' Object used for matching objects file
    Set ObjLogMSDE = ObjFPC.GetContainingArray.LogViewer.LogContentMSDE
    
    ' Object used for logs filtering
    Set ObjFilterExpressions = CreateObject( "Fpc.FpcFilterExpressions" )
    
    ' Witch kins of logs to analyse (log viewer)
    ObjFilterExpressions.FilterType = FpcLogViewerFilter
    
    ' Filter constraints
    ObjFilterExpressions.AddDateFilter FpcFilterByLogTime, FpcOnOrAfter, (Date() - iNbDays)
    ' OnOrBefore is used to search between 2 dates, in this case search
    ' will be performed from some days to now.
    'ObjFilterExpressions.AddDateFilter FpcFilterByLogTime, FpcOnOrBefore, Date()
    ObjFilterExpressions.AddStringFilter FpcFilterByUrl, FpcContains, strUrlRacine

    If Err.Number = 0 Then
        ' Execute Query with filters
    	ObjLogMSDE.ExecuteQuery ObjFilterExpressions, iNbRecords

	' Load records
	For Each LogMSDERecord In ObjLogMSDE
	    strLogURLString = LogMSDERecord.URL
	    iCounter = iCounter + 1
	    bObjectOk = TRUE

            ' Suppress not applicable lines for caching
            If strLogURLString = strUrlRacine Then
                bObjectOk = FALSE
            End If
            If InStr(strLogURLString, "/*") Then
                bObjectOk = FALSE
            End If
            If InStr(strLogURLString, " ") Then
                bObjectOk = FALSE
            End If
            If InStr(strLogURLString, ".asp?") Then
                bObjectOk = FALSE
            End If
            If InStr(strLogURLString, ".aspx?") Then
                bObjectOk = FALSE
            End If
            If Right(strLogURLString, 1) = "/" Then
                bObjectOk = FALSE
            End If
            If LCase(Right(strFileCache, 4)) <> "none" Then
	        If LineExistInFile(strFileCache, strLogURLString) > 0 Then
	            bObjectOk = FALSE
                End If
            Else
                If lineExistInArray(arrObjectInLog, strLogURLString) > 0 Then
	            bObjectOk = FALSE
                End If
            End If

	    If bObjectOk Then
	        iDistinctObject = iDistinctObject + 1
	        If LCase(Right(strFileCache, 4)) <> "none" Then
                    call LogAction(strFileCache, strLogURLString, FALSE)
                Else
                    arrObjectInLog(iDistinctObject) = strLogURLString
                End If
            End If
        Next
        
        ' Log in report file
        call LogAction(strFileLog, "    Racine : " & strUrlRacine & _
            VbCrLf & "    Nb Records Max : " & iNbRecords & _
            VbCrLf & "    Nb Days : " & iNbDays & _
            VbCrLf & "    Nb Records " & iCounter & _
            VbCrLf & "    Nb Objects " & iDistinctObject & _
            VbCrLf & "    Nb Filters : " & ObjFilterExpressions.Count, FALSE)

        If bDisplay And iDistinctURL < 3 Then
            WScript.Echo "Racine : " & strUrlRacine & _
                VbCrLf & "Nb Records Max : " & iNbRecords & _
                VbCrLf & "Nb Days : " & iNbDays & _
                VbCrLf & "Nb Records " & iCounter & _
                VbCrLf & "Nb Objects " & iDistinctObject & _
                VbCrLf & "Nb Filters : " & ObjFilterExpressions.Count
        End If
    Else
        iErrorNumber = Err.Number
        strErrorDescription = Err.Description

        strLogString = "    Error nb : " & iErrorNumber & _
            VbCrLf & strUrlRacine & _
            VbCrLf & "    " & strErrorDescription
        call LogAction(strFileLog, strLogString, FALSE)
        If bDisplay Then
            WScript.Echo strLogString
        End If
    End If

    ObjLogMSDE.EndQuery
    Set ObjFilterExpressions = Nothing
End Sub

'------------------------------------------------------------
'    Purpose : Update given objects in cache
'    Arguments : FPC (ISA) Root Object
'                string URL to update in cache
'                string URL Name in cache
'                string log file name
'    Global : Error Code - 0 if ok
'             Error Description
'
'    If URL to update is Empty, then delete URL Name in cache
'------------------------------------------------------------
Sub CacheUpdate(ObjFPC, strUrlUpdate, strUrlClient, strLogFile)
    Const fpcFetchTtlIfNone      = 1
    Const fpcFetchTtlOverride    = 2
    Const fpcFetchSynchronous    = 4
    Const fpcFetchNoArrayRouting = 8
    Const fpcFetchForceCache     = 16
    Const fpcFetchDynamicCache   = 32

    Dim objCache
    Dim customTTL
    Dim strLogString

    customTTL = 10
    iErrorNumber = 0

    On Error Resume Next

    Set objCache = objFPC.GetContainingArray.Cache.CacheContents

    ' Fetch URL source and to store it in the cache under the given name
    ' If source URL is empty, remove URL cache Name
    If strUrlUpdate = "" Then
        objCache.FetchURL strUrlUpdate, strUrlClient, 0, 0
    Else
        objCache.FetchURL strUrlUpdate, strUrlClient, _
            customTTL, fpcFetchSynchronous
    End If

    If Err.Number <> 0 Then
        iErrorNumber = Err.Number
        strErrorDescription = Err.Description

        strLogString = "    Error nb : " & iErrorNumber & _
            VbCrLf & strUrlUpdate & _
            VbCrLf & "    " & strErrorDescription
    Else
        strLogString = strUrlUpdate
    End If

    ' log in report file
    call LogAction(strLogFile, strLogString, FALSE)

    set objCache = Nothing
End Sub

'===========================================================================
' Const values for logs analysis (Globals)
'===========================================================================

    ' FpcFilterType
    Const FpcNoFilterType      = 0
    Const FpcSessionsFilter    = 1
    Const FpcLogViewerFilter   = 2
    
    ' FpcFilterCondition
    Const FpcNoCondition       = &h00000000
    Const FpcEqual             = &h00000001
    Const FpcNotEqual          = &h00000002
    Const FpcGreaterOrEqual    = &h00000004
    Const FpcLessOrEqual       = &h00000008
    Const FpcOnOrAfter         = &h00000010
    Const FpcOnOrBefore        = &h00000020
    Const FpcContains          = &h00000040
    Const FpcLiveData          = &h00000080
    Const FpcLastHour          = &h00000100
    Const FpcLast24Hours       = &h00000200
    Const FpcLast7days         = &h00000400
    Const FpcLast30days        = &h00000800
    Const FpcNotContains       = &h00001000

    ' FpcFilterCriteria
    Const FpcNoFilter                   = 0
    Const FpcFilterByClientIP           = 1
    Const FpcFilterByClientUserName     = 2
    Const FpcFilterByClientAgent        = 3
    Const FpcFilterByAuthenticated      = 4
    Const FpcFilterByService            = 5
    Const FpcFilterByServername         = 6
    Const FpcFilterByReferringServer    = 7
    Const FpcFilterByDestHost           = 8
    Const FpcFilterByDestHostIP         = 9
    Const FpcFilterByProtocol           = 10
    Const FpcFilterByTransport          = 11
    Const FpcFilterByOperation          = 12
    Const FpcFilterByUrl                = 13
    Const FpcFilterByRule               = 14
    Const FpcFilterByFilterInfo         = 15
    Const FpcFilterByMimetype           = 16
    Const FpcFilterByObjectSource       = 17
    Const FpcFilterByLogTime            = 18
    Const FpcFilterByCacheInfo          = 19
    Const FpcFilterByErrorInfo          = 20
    Const FpcFilterByDestHostPort       = 21
    Const FpcFilterBySrcPort            = 22
    Const FpcFilterByLogType            = 23
    Const FpcFilterBySessionType        = 24
    Const FpcFilterByActivation         = 25
    Const FpcFilterByClientComputerName = 26
    Const FpcFilterBySourceNetwork      = 27
    Const FpcFilterByDestinationNetwork = 28
    Const FpcFilterBySourceProxy        = 29
    Const FpcFilterByDestinationProxy   = 30
    Const FpcFilterByAction             = 31
    Const FpcFilterByBidirectional      = 32
    Const FpcFilterByInterface          = 33
    Const FpcFilterByHeader             = 34
    Const FpcFilterByPayload            = 35
    Const FpcFilterProcessingTime       = 36
    Const FpcFilterByResultCode         = 37
    Const FpcFilterByBytesSent          = 38
    Const FpcFilterByBytesReceived      = 39
    Const FpcFilterByApplicationName    = 40
    Const FpcFilterByOriginalClientIP   = 41
    Const FpcFilterByHTTPCode           = 42
    
    ' FpcAction
    Const FpcActionNotLogged              = 0
    Const FpcActionBind                   = 1
    Const FpcActionListen                 = 2
    Const FpcActionGHBN                   = 3
    Const FpcActionGHBA                   = 4
    Const FpcActionRedirectBind           = 5
    Const FpcActionEstablish              = 6
    Const FpcActionTerminate              = 7
    Const FpcActionDenied                 = 8
    Const FpcActionAllowed                = 9
    Const FpcActionFailed                 = 10
    Const FpcActionIntermediate           = 11
    Const FpcActionSuccessfulConnection   = 12
    Const FpcActionUnsuccessfulConnection = 13
    Const FpcActionDisconnection          = 14
    Const FpcActionUserclearedQuarantine  = 15
    Const FpcActionQuarantinetimeout      = 16

